package org.foo

import java.util.jar.Attributes
import java.util.jar.JarFile
import java.util.jar.Manifest

import org.gradle.api.GradleException
import org.gradle.api.Project
import org.gradle.api.Plugin
import org.gradle.api.artifacts.ResolvedArtifact
import org.gradle.api.artifacts.ResolvedConfiguration

class HelloPlugin implements Plugin<Project> {
	
	public static final String MODULE_GROUP = 'org.foo'
	public static final String MODULE_NAME = 'helloplugin'
	
	protected static final BUILD_NUMBER_MANIFEST_ATTRIBUTE = 'Build-Number'
	
	public void apply(Project project) {
		//find the hello plugin's jar and read its build number from manifest:
		ResolvedArtifact helloPlugin = findHelloPluginArtifact(project)
		
		assert helloPlugin : "Could not find HelloPlugin's artifact in buildscript's classpath"
		
		String buildNumber = getBuildNumber(project, helloPlugin)
		println "Hello plugin version=${helloPlugin.moduleVersion.id.version}, buildNumber=$buildNumber applied to project '$project.name'."
		
		project.convention.plugins.hello = new HelloPluginConvention(buildNumber)
	}
	
	private ResolvedArtifact findHelloPluginArtifact(Project targetProject) {
		while (targetProject) {
			ResolvedConfiguration cp = targetProject.buildscript.configurations.classpath.resolvedConfiguration
			ResolvedArtifact helloPluginArtifact = cp.resolvedArtifacts.find {
				it.moduleVersion.id.group == MODULE_GROUP && it.moduleVersion.id.name == MODULE_NAME
			}
			if (helloPluginArtifact) {
				return helloPluginArtifact
			}
			targetProject = targetProject.parent
		}
		return null
	}
	
	private String getBuildNumber(Project project, ResolvedArtifact artifact) {
		File artifactFile = artifact.getFile()
		
		assert artifactFile : "Artifact '$artifact' does not define a file."
		assert artifactFile.exists() : "Artifact '$artifact' does not exist: $artifactFile.absolutePath"

		JarFile jarFile = null
		try {
			jarFile = new JarFile(artifactFile)
			Manifest manifest = jarFile.getManifest()
			
			assert manifest : "Artifact '$artifact' does not contain a manifest: $artifactFile.absolutePath"
			
			Attributes attr = manifest.getMainAttributes()
			String buildNumber = attr.getValue(BUILD_NUMBER_MANIFEST_ATTRIBUTE)
			
			assert buildNumber : "Artifact '$artifact' does not contain a '$BUILD_NUMBER_MANIFEST_ATTRIBUTE' manifest attribute: $artifactFile.absolutePath"
			
			return buildNumber
		}
		catch (IOException e) {
			throw new GradleException("Failed to read '$BUILD_NUMBER_MANIFEST_ATTRIBUTE' manifest attribute of artifact '$artifact' at: $artifactFile.absolutePath", e)
		}
		finally {
			if (jarFile != null) {
				try {
					jarFile.close()
				}
				catch (IOException e) {
					project.logger.warn("Failed to close artifact jar file: $artifactFile.absolutePath", e)
				}
			}
		}
	}
}